#include "draw.h"
#include "config.h"

char *colornames[16] = { 
    "Black",
    "Blue",
    "ForestGreen",
    "SkyBlue",
    "IndianRed",
    "HotPink",
    "Brown",
    "LightGrey",
    "DimGrey",
    "DeepSkyBlue",
    "Green",
    "Cyan",
    "Tomato",
    "Magenta",
    "Yellow",
    "White"
};

Colormap colormap;
Pixel    colors[NUM_COLORS];
Pixmap   stipplemap[16];
Pixmap   stipplePixmap[16];
int      monoFlag;
int      cColor = 4;
int      cBackColor = 0;

void HLStoRGB(H,L,S, R,G,B)
int H;
double L,S;
unsigned short *R,*G,*B;
{
    double r, g, b, m, M;
    
    if (L >= 0.5)
      M = L*(1+S);
    else
      M = L+S-L*S;
    m = 2*L-M;
    
    /* Compute R */
    if (H<60)
      r = m+(M-m)*H/60;
    else if (H<180)
      r = M;
    else if (H<240)
      r = m+(M-m)*(240-H)/60;
    else if (H<360)
      r = m;
    
    /* Compute G */
    if (H<120)
      g = m;
    else if (H<180)
      g = m+(M-m)*(H-120)/60;
    else if (H<300)
      g = M;
    else if (H<360)
      g = m+(M-m)*(360-H)/60;
    
    /* Compute B */
    if (H<60)
      b = M;
    else if (H<120)
      b = m+(M-m)*(120-H)/60;
    else if (H<240)
      b = m;
    else if (H<300)
      b = m+(M-m)*(H-240)/60;
    else if (H<360)
      b = 1.0;
  
    *R = r*65535;
    *G = g*65535;
    *B = b*65535;
}

/* Sample pallete for 256 colors. First 16 are the same as the 16 color
 * pallete. So this adds 240 colors. Someone please come up with better
 * colors.  -MS
 */
PalletteMake()
{
    XColor rgbcolor;
    int i, hue;

    for (i = 0; i < 216; i++) {
       rgbcolor.red = (i%6)*(255/6) << 8;
       rgbcolor.green = ((i/6)%6)*(255/6) << 8;
       rgbcolor.blue = ((i/36)%6)*(255/6) << 8; 
       rgbcolor.flags = DoRed | DoGreen | DoBlue;
       colors[i+16] = rgbcolor.pixel = (Pixel) i+16;
       XStoreColor(dpy, colormap, &rgbcolor);
    }
    for (i = 0; i < 24; i++) {
       hue = i*(360.0/24.0);
       HLStoRGB(hue, 0.5, 1.0, 
		&rgbcolor.red, &rgbcolor.green, &rgbcolor.blue);
       rgbcolor.flags = DoRed | DoGreen | DoBlue;
       colors[i+16+216] = rgbcolor.pixel = (Pixel) i+16+216;
       XStoreColor(dpy, colormap, &rgbcolor);
    }
}

int isMono()
{
    return monoFlag;
}

Pixmap GetPixmap(i)
int i;
{
    return stipplePixmap[i];
}

Pixmap GetStipple(i)
int i;
{
    return stipplemap[i];
}

Pixel GetColor(i)
int i;
{
    if (monoFlag) {
	if (i == 0)
	  return BlackPixel(dpy, DefaultScreen(dpy));
	return WhitePixel(dpy, DefaultScreen(dpy));
    }
    return colors[i];
}

void SetGCToColor(w, gc, color, type)
Widget w;
GC     gc;
int    color;
int    type;
{
    if (monoFlag) {
	if (type != XOR) 
	    XSetStipple(XtDisplay(w), gc, GetStipple(color));
	XSetForeground(XtDisplay(w), gc, GetColor(0));
	XSetBackground(XtDisplay(w), gc, GetColor(1));
    }
    else {
	if ((type == XOR) && (cBackColor != color))
	  XSetForeground(XtDisplay(w), gc, 
			 colors[cBackColor] ^ colors[color]);
	else 
	  XSetForeground(XtDisplay(w), gc, colors[color]);
    }
}

void SetGCColor(w, gc, type)
Widget w;
GC     gc;
int    type;
{
   SetGCToColor(w, gc, cColor, type);
}

MakeColormap()
{
    XColor dummy;
    static int try=0;
    Window rootw;

    rootw = RootWindow(dpy, DefaultScreen(dpy));
#ifdef COLORS_256
    colormap = XCreateColormap(dpy, rootw, 
                               DefaultVisual(dpy, DefaultScreen(dpy)),
                               AllocAll); 
#else
    if (try) {
       colormap = XCreateColormap(dpy, rootw, 
                                  DefaultVisual(dpy, DefaultScreen(dpy)),
                                  AllocNone); 
       XAllocNamedColor(dpy, colormap, "White", &dummy, &dummy);
       XAllocNamedColor(dpy, colormap, "Black", &dummy, &dummy);
    }
    else {
       colormap= DefaultColormap(dpy, DefaultScreen(dpy)); 
       try++;
    }
#endif
}

MakeMono()
{
    Window    rootw;
    Pixmap    temp;
    char      fname[256];
    unsigned  dummy_width, dummy_height;
    int       depth, i, dummy_xhot, dummy_yhot;

    colormap= DefaultColormap(dpy, DefaultScreen(dpy)); 
    depth = DisplayPlanes(dpy, DefaultScreen(dpy)); 
    rootw = RootWindow(dpy, DefaultScreen(dpy));
    for (i=0; i<16; i++) {
	strcpy(fname, STIPPLEDIR);
	strcat(fname, colornames[i]);
	strcat(fname, ".xbm");
	if (XReadBitmapFile(dpy, rootw, fname,
			    &dummy_width, &dummy_height,
			    &(stipplemap[i]),
			    &dummy_xhot, &dummy_yhot) != BitmapSuccess)
	   error("muddraw: Error reading bitmap file.");
        /* This is for mono on non-mono displays */
        if (depth != 1) {
            stipplePixmap[i] = XCreatePixmap(dpy, rootw, 
					  dummy_width, dummy_height, depth);
	    XCopyPlane(dpy, stipplemap[i], stipplePixmap[i],
		       DefaultGC(dpy, DefaultScreen(dpy)),
		       0, 0, dummy_width, dummy_height, 0, 0, 1);
	}
	else 
            stipplePixmap[i] = stipplemap[i];
    }
}

int Make16()
{
    XColor hardwarecolor, rgbcolor;
    int i;
    
    for (i=0; i<16; i++) {
	if(!XAllocNamedColor(dpy, colormap, colornames[i], 
			     &rgbcolor, &hardwarecolor)) {
           printf("Failed to alloc color %s.\n", colornames[i]);
	   return 0;
        }
	colors[i] = hardwarecolor.pixel;
    }
    return 1;
}

Make256()
{
    XColor rgbcolor, hardwarecolor;
    int i;
    
    for (i=0; i<16; i++) {
	XLookupColor(dpy,colormap,
	             colornames[i],&rgbcolor,&hardwarecolor);
        hardwarecolor.flags = DoRed | DoGreen | DoBlue;
	if (i == 15)           /* Store white as pixel 0 */
	   colors[i] = hardwarecolor.pixel = 0;
	else if (i == 0)       /* Store black as pixel 1 */ 
	   colors[i] = hardwarecolor.pixel = 1;
        else 
	   colors[i] = hardwarecolor.pixel = i+2; /* first two are reserved */
        XStoreColor(dpy, colormap, &hardwarecolor);
    }
    /* this is just a sample pallette */
    PalletteMake();
}

MakeColors()
{
    MakeColormap();
    if (monoFlag || DisplayPlanes(dpy, DefaultScreen(dpy))==1) { 
	monoFlag = 1;
	/* Monochrome mode */
	MakeMono();
    }
    else {
#ifdef COLORS_256
	Make256();
#else
	if (!Make16()) {
           MakeColormap();    /* Default is full, Use a seperate colormap */
	   if (!Make16()) {
	      printf("draw: Could not allocate all %d colors.\n", NUM_COLORS);
	      monoFlag = 1;
	      MakeMono();
           }
        }
#endif
    }
}

SetColormap(w)
Widget w;
{
    Arg wargs[1];
    int n;
    
    n=0;
    XtSetArg(wargs[n], XtNcolormap, colormap); n++;
    XtSetValues(w, wargs, n);
}

GetColormap(w)
Widget w;
{
    Colormap colormap;
    Arg wargs[1];
    int n;

    n=0;
    XtSetArg(wargs[n], XtNcolormap, &colormap); n++;
    XtGetValues(w, wargs, n);
}

void ChangeBackground(w)
Widget w;
{
    Arg wargs[1];
    int n;

    n=0;
    if (monoFlag) {
       XtSetArg(wargs[n], XtNbackgroundPixmap, GetPixmap(cColor)); n++;
    }
    else {
       XtSetArg(wargs[n], XtNbackground, GetColor(cColor)); n++;
    }
    XtSetValues(w, wargs, n);
    cBackColor = cColor;
}
