#include "config.h"
#include "draw.h"

drawData  data;

extern int    cColor;
extern int    cBackColor;
extern Widget canvas, polyNum;

void CreateDashGC(w)
Widget  w;
{
  XGCValues values;

  values.line_style = LineOnOffDash;
  values.function = GXxor;
  data.dashGc = XtGetGC(w, GCFunction | GCLineStyle, &values);
}

void CreateSolidXorGC(w)
Widget  w;
{
  XGCValues values;

  values.line_style = LineSolid;
  values.function = GXxor;
  data.solidXorGc = XtGetGC(w, GCFunction | GCLineStyle, &values);
}

void CreateSolidGC(w)
Widget  w;
{
  XGCValues values;

  if (isMono()) {
      values.line_style = LineSolid;
      values.function = GXcopy;
      values.fill_style = FillOpaqueStippled;
      data.solidGc = XtGetGC(w, GCFunction | GCLineStyle | GCFillStyle,
			     &values);
  }
  else {
      values.line_style = LineSolid;
      values.function = GXcopy;
      data.solidGc = XtGetGC(w, GCFunction | GCLineStyle, &values);
  }
}

void CreateGCs(w)
Widget  w;
{
  Arg       wargs[2];
  
  CreateDashGC(w);
  CreateSolidXorGC(w);
  CreateSolidGC(w);
}

void HandlePress(w, client_data, event)
Widget     w;
caddr_t    client_data;
XEvent     *event;
{
   if (data.doingPoly)
      UpdateRubber(w, event);
   else if (data.doingRect)
      UpdateRect(w, event);
   else if (data.doingElipse)
      UpdateElipse(w, event);
   else if (data.doingMove)
      UpdateMove(w, event);
   else if (data.doingCopy)
      UpdateCopy(w, event);
   else if (data.doingPointAdjust) 
      UpdatePointAdjust(w, event);
   else if (data.cPrimitive == 9)
      Delete(w,event);
   else if (data.cPrimitive == 8)
      Forward(w,event);
   else if (data.cPrimitive == 7)
      Back(w,event);
   else if (data.cPrimitive == 6)
      StartPointAdjust(w, event);
   else if (data.cPrimitive == 5)
      StartCopy(w, event);
   else if (data.cPrimitive == 4)
      StartMove(w, event);
   else if (data.cPrimitive == 3)
      StartElipse(w, event);  
   else if (data.cPrimitive == 2)
      StartRect(w, event);
   else if (data.cPrimitive == 1)
      StartRubber(w, event);
}

void HandleMove(w, client_data, event)
Widget     w;
caddr_t    client_data;
XEvent     *event;
{
   if (data.doingPoly)
      MoveRubber(w, event);
   if (data.doingRect)
      MoveRect(w, event);
   if (data.doingElipse)
      MoveElipse(w, event);
   if (data.doingMove)
      MoveMove(w, event);
   if (data.doingCopy)
      MoveCopy(w, event);
   if (data.doingPointAdjust)
      MovePointAdjust(w, event);
}

void ButtonCallback(w, actionNum, call_data)
Widget     w;
int        actionNum;
caddr_t    call_data;
{
   void ClearPicture(), RedrawPicture();
   int i;

   switch (actionNum) {
      case 1:  
         exit(0);
	 break;
      case 2:  
	 data.cPoly = 0;
	 UpdateUndo();
	 SetPolyNumLabel();
         ClearPicture();
         RedrawPicture();
	 break;
      case 3:
	 GetFileName(1);
	 break;
      case 4:
	 GetFileName(2);
	 break;
      case 5:
	 ChangeBackground(canvas);
	 break;
      case 6:
	 if (data.undo.picpointer > 0) {
	    data.undo.picpointer--;
	    UpdateScene();
	    ClearPicture();
	    RedrawPicture();
	    SetPolyNumLabel();
         }
	 break;
      case 7:
	 if (data.undo.picpointer < data.undo.numundos) {
	    data.undo.picpointer++;
	    UpdateScene();
	    ClearPicture();
	    RedrawPicture();
	    SetPolyNumLabel();
	 }
	 break;
      default:
	break;
   }
}

void ColorCallback(w, color, call_data)
Widget     w;
int        color;
caddr_t    call_data;
{
   cColor = color; 
}

void PrimitiveCallback(w, prim, call_data)
Widget     w;
int        prim;
caddr_t    call_data;
{
   data.cPrimitive = prim; 
}

void ExposeCallback(w, color, call_data)
Widget     w;
int        color;
caddr_t    call_data;
{
   void RedrawPicture();

   RedrawPicture();
}

void InitDrawData()
{
   data.doingPoly = FALSE;
   data.doingRect = FALSE;
   data.doingElipse = FALSE;
   data.doingMove = FALSE;
   data.doingCopy = FALSE;
   data.doingPointAdjust = FALSE;
   data.cPrimitive = 1;
   data.cPoly = 0;
   data.undo.numundos=0;
   data.undo.picpointer=0;
   data.undo.pastpic[0].cPoly=0;
}

void ClearPicture()
{
   SetGCToColor(canvas, data.solidGc, cBackColor, COPY);
   XFillRectangle(XtDisplay(canvas), XtWindow(canvas), 
                  data.solidGc, 0, 0, WIDTH, HEIGHT);
}

void DrawAxis()
{
   int axisColor;

   axisColor = AXIS_COLOR;
   if (axisColor == cBackColor)
      axisColor = (axisColor++)%NUM_COLORS;
   SetGCToColor(canvas, data.solidGc, axisColor, COPY);
    
   XDrawLine(XtDisplay(canvas), XtWindow(canvas), data.solidGc, 
             0, HEIGHT >> 1, WIDTH, HEIGHT >> 1);
   XDrawLine(XtDisplay(canvas), XtWindow(canvas), data.solidGc, 
             WIDTH >> 1, 0, WIDTH >> 1, HEIGHT);
}

void DoOutline(i)
int i;
{
   XDrawLines(XtDisplay(canvas), XtWindow(canvas), 
              data.solidXorGc, data.pic[i].points,
	      data.pic[i].size, CoordModeOrigin);
}

void DrawXorPoly(i)
int i;
{
   SetGCToColor(canvas, data.solidXorGc, data.pic[i].color, XOR);
   XFillPolygon(XtDisplay(canvas), XtWindow(canvas), 
                data.solidXorGc, data.pic[i].points,
		data.pic[i].size, 0, 0);
}

void DrawPoly(i)
int i;
{
   RoundPoints(i);
   SetGCToColor(canvas, data.solidGc, data.pic[i].color, COPY);
   XFillPolygon(XtDisplay(canvas), XtWindow(canvas), 
                data.solidGc, data.pic[i].points,
		data.pic[i].size, 0, 0);
}

/* Redraws the whole scene except for one poly */
void RedrawExcept(poly)
int poly;
{
   int i;

   DrawAxis();
   for (i = 0; i < data.cPoly; i++) {
      if (i != poly)
         DrawPoly(i);
   }
}

void RedrawPicture()
{
   int i;

   DrawAxis();
   for (i = 0; i < data.cPoly; i++) {
      DrawPoly(i);
   }
}

SetPolyNumLabel()
{
   Arg    wargs[2];
   char   name[40];
   int    n;

   sprintf(name, "Polygons: %d", data.cPoly);
   n = 0;
   XtSetArg (wargs[n], XtNlabel, name);              n++;
   XtSetValues(polyNum, wargs, n);
}
