#include <stdio.h>
#include "config.h"
#include "draw.h"
#include "file.h"

extern drawData  data;
extern char *colornames[];
int error;

/* This rounds the point down to 0-255 then back up
 * suitable for display
 */
RoundPoints(i)
int i;
{
   int j;

   for (j = 0; j < data.pic[i].size; j++) {
      data.pic[i].points[j].x = 
	  convertIn(convertOut(data.pic[i].points[j].x, X), X);
      data.pic[i].points[j].y = 
	  convertIn(convertOut(data.pic[i].points[j].y, Y), Y);
   }
}

int FindGraph(f)
FILE *f;
{
   static char s[] = "graph()\0";
   int         i, found = 0;
   char        c;

   while (((c = fgetc(f)) != EOF) && (!found)) {
      i = 0;
      while (c == s[i]) {
         c = fgetc(f);
         i++;
         if (s[i] == '\0') {
	    found = 1;
	    break;
         }
      }
   }
   return found;
}

CheckIntegrity()
{
  int i, j;

  if (data.cPoly > MAXDESCSIZE) {
     printf("Number of polys in scene out of bounds.\n");
     error=1;
     return;
  }
  for (i=0; i<data.cPoly; i++) {
    if (data.pic[i].size > MAXPOINTPOLY) {
      printf("Size out of bounds in polygon #%d.\n", i);
      error=1;
      return;
    }
    if (data.pic[i].color > NUM_COLORS) {
      printf("Color out of bounds in polygon #%d.\n", i);
      error=1;
      return;
    }
    for (j=0; j < data.pic[i].size; j++) 
      if ((data.pic[i].points[j].x < 0) 
	  || (data.pic[i].points[j].x > WIDTH)
	  || (data.pic[i].points[j].y < 0) 
	  || (data.pic[i].points[j].y > HEIGHT)) {
         printf("Point #%d in poly #%d is out of bounds.\n", j, i);
         error=1;
         return;
      }
  }
}

int ColornameToInt(s)
char *s;
{
   int i;

   for(i = 0; i < 16; i++) {
      if (strcmp(s, colornames[i]) == 0)
	 break;
   }
   if (i == 16)
      return BAD_COLORNAME;
   return i;
}

int convertOut(value, cord)
{
   if (cord == X)
      return value/3;
   if (cord == Y)
      return (HEIGHT - value)/2;
}

int convertIn(value, cord)
{
   if (cord == X)
      return value*3;
   if (cord == Y)
      return HEIGHT - value*2;
}

int GetAChar(f)
FILE *f;
{
   int c;

   c = fgetc(f);
   if (c == EOF) 
      error = 1;
   return c;
}

int ishex(c)
char c;
{
   return ((c>='0'&&c<='9')||(c>='A'&&c<='F')||(c>='a'&&c<='f'));
}

unsigned char HexToInt(hi, lo)
int lo,hi;
{
  unsigned char b;

  if (hi > 57)
    hi = hi - 55;
  else
    hi = hi - 48;
  if (lo > 57)
    lo = lo - 55;
  else
    lo = lo - 48;
  b = (unsigned char) hi*16 + lo;
  return b;
}

void ToHex(b, hi, lo)
unsigned char b, *hi, *lo;
{
  *hi = b / 16;
  *lo = b % 16;
  if (*hi > 9)
    *hi = *hi + 55;
  else
    *hi = *hi + 48;
  if (*lo > 9)
    *lo = *lo + 55;
  else
    *lo = *lo + 48;
}

void fputhex(b,f)
unsigned char b;
FILE *f;
{
  unsigned char lo,hi;

  hi = b / 16;
  lo = b % 16;
  if (hi > 9)
    hi = hi + 55;
  else
    hi = hi + 48;
  if (lo > 9)
    lo = lo + 55;
  else
    lo = lo + 48;
  fputc(hi,f);
  fputc(lo,f);
}

unsigned char fgethex(f)
FILE *f;
{
  unsigned char b;
  int lo,hi;

  hi = fgetc(f);
  lo = fgetc(f);
  if ((hi == EOF && lo == EOF) || (!ishex(hi) || !ishex(lo))) {
     error=1;
     return 0;
  }
  if (hi > 57)
    hi = hi - 55;
  else
    hi = hi - 48;
  if (lo > 57)
    lo = lo - 55;
  else
    lo = lo - 48;
  b = (unsigned char) hi*16 + lo;
  return b;
}

int findType(filename)
char *filename;
{
   char *p;

   if (strlen(filename) > 4) {
      p = filename+strlen(filename)-4;
      if (strcmp(".ppf", p) == 0)
         return PPF;
      if (strcmp(".bsx", p) == 0)
         return BSX;
      if (strcmp(".mud", p) == 0)
         return MUD;
      if (strcmp(".xpf", p) == 0)
         return XPF;
   }
   return BAD_FILE_TYPE;
}

void save_xpf(filename)
char *filename;
{
  FILE *f;
  int i, j, x, y;

  f = fopen(filename,"w");
  if (!f) {
    error=1;
    return;
  }
  printf("Writing %d polygons to file.\n",data.cPoly);
  fprintf(f, "%d\n", data.cPoly);
  for (i=0; i<data.cPoly; i++) {
    fprintf(f, "%d ", data.pic[i].size);
    fprintf(f, "%s ", colornames[data.pic[i].color]);
    for (j=0; j < data.pic[i].size; j++) {
      x = convertOut(data.pic[i].points[j].x, X);
      y = convertOut(data.pic[i].points[j].y, Y);
      fprintf(f, "(%d,%d) ", x, y);
    }
    fprintf(f, "\n");
  }
  fclose(f);
}

void save_bsx(filename)
char *filename;
{
  FILE *f;
  int i, j;

  f = fopen(filename,"w");
  if (!f) {
    error=1;
    return;
  }
  printf("Writing %d polygons to file.\n",data.cPoly);
  fputhex(data.cPoly,f);
  for (i=0; i<data.cPoly; i++) {
    fputhex(data.pic[i].size,f);
    fputhex(data.pic[i].color,f);
    for (j=0; j < data.pic[i].size; j++) {
      fputhex(convertOut(data.pic[i].points[j].x, X),f);
      fputhex(convertOut(data.pic[i].points[j].y, Y),f);
    }
  }
  fclose(f);
}

void save_ppf(filename)
char *filename;
{
  FILE *f;
  int i, j;

  f = fopen(filename,"wb");
  if (!f) {
    error=1;
    return;
  }
  printf("Writing %d polygons to file.\n",data.cPoly);
  fputc(data.cPoly,f);
  for (i=0; i<data.cPoly; i++) {
    fputc(data.pic[i].size,f);
    fputc(data.pic[i].color,f);
    for (j=0; j < data.pic[i].size; j++) {
      fputc(convertOut(data.pic[i].points[j].x, X),f);
      fputc(convertOut(data.pic[i].points[j].y, Y), f);
    }
  }
  fclose(f);
}

void save_mud(filename)
char *filename;
{
  FILE *f;
  unsigned char *s;
  int cnt, i, j;

  f = fopen(filename,"w");
  if (!f) {
    error=1;
    return;
  }
  s = (unsigned char *) 
	 malloc(sizeof(char)*(3+MAXDESCSIZE*(4+4*MAXPOINTPOLY)));
  cnt = 0;
  ToHex(data.cPoly, &(s[cnt]), &(s[cnt+1]));
  cnt += 2;
  for (i=0; i<data.cPoly; i++) {
    ToHex(data.pic[i].size, &(s[cnt]), &(s[cnt+1]));
    ToHex(data.pic[i].color, &(s[cnt+2]), &(s[cnt+3]));
    cnt += 4;
    for (j=0; j < data.pic[i].size; j++) {
      ToHex(convertOut(data.pic[i].points[j].x, X),&(s[cnt]), &(s[cnt+1]));
      ToHex(convertOut(data.pic[i].points[j].y, Y),&(s[cnt+2]), &(s[cnt+3]));
      cnt += 4;
    }
  }
  s[cnt] = '\0'; 
  
  printf("Writing %d polygons to file.\n",data.cPoly);
  fprintf(f, "/* MudDraw (Version: %s) */\n", VERSION);
  fprintf(f, "#include \"/2nd_graph.h\"\n");
  fprintf(f, "string graph() {\n");
  fprintf(f, "      return \"");
  cnt=0;
  while (s[cnt] != '\0') {
     if(!(cnt++%64)) fprintf(f,"\"+\n      \"");
     fprintf(f,"%c",s[cnt-1]);
  }
  fprintf(f,"\";\n}\n");
  fclose(f);
  free(s);
}

void save_file(filename)
char *filename;
{
   int type;

   error = 0;
   if (filename == NULL) {
      printf("No file name, file not saved.\n");
      return;
   }
   type = findType(filename);
   switch (type) {
      case PPF:
	save_ppf(filename);
	break;
      case BSX:
	save_bsx(filename);
	break;
      case MUD:
	save_mud(filename);
	break;
      case XPF:
	save_xpf(filename);
	break;
      case BAD_FILE_TYPE:
      default:
        printf("Illegal file extention, file not saved.\n");
   }
   if (error) {
      printf("Error saving file.\n");
      data.cPoly = 0;
      SetPolyNumLabel();
   }
}

void load_ppf(filename) 
char *filename;
{
  FILE *f;
  int i, j;

  f = fopen(filename,"rb");
  if (!f) {
    printf("Cannot read file, Sorry!\n");
    error=1;
    return;
  }
  data.cPoly = GetAChar(f);
  printf("Reading %d polygons from file.\n",data.cPoly);
  if (data.cPoly < 0) return;
  for (i=0; i < data.cPoly; i++) {
    data.pic[i].size = GetAChar(f);
    data.pic[i].color = GetAChar(f);
    for (j=0; j<data.pic[i].size; j++) {
      data.pic[i].points[j].x = convertIn(GetAChar(f), X);
      data.pic[i].points[j].y = convertIn(GetAChar(f), Y);
    }
  }
  fclose(f);
}

void load_bsx(filename)
char *filename;
{
  FILE *f;
  int i, j;

  f = fopen(filename,"r");
  if (!f) {
    printf("Cannot read file, Sorry!\n");
    error=1;
    return;
  }
  data.cPoly = fgethex(f);
  printf("Reading %d polygons from file.\n",data.cPoly);
  for (i=0; i<data.cPoly; i++) {
    data.pic[i].size = fgethex(f);
    data.pic[i].color = fgethex(f);
    for (j=0; j < data.pic[i].size; j++) {
      data.pic[i].points[j].x = convertIn(fgethex(f), X);
      data.pic[i].points[j].y = convertIn(fgethex(f), Y);
    }
  }
  fclose(f);
}

void load_xpf(filename)
char *filename;
{
  FILE *f;
  char *colorString;
  int i, j, x, y;

  f = fopen(filename,"r");
  if (!f) {
    printf("Cannot read file, Sorry!\n");
    error=1;
    return;
  }
  colorString = (char *) malloc(sizeof(char)*MAX_COLORNAME_LENGTH);
  if (fscanf(f, "%d", &(data.cPoly)) == EOF) {
    printf("File does not exist or invalid format.");
    error = 1;
    return;
  }
  printf("Reading %d polygons from file.\n",data.cPoly);
  for (i=0; i<data.cPoly; i++) {
    if (fscanf(f, "%d", &(data.pic[i].size)) == EOF) {
      printf("Invalid number of segments near line %d.\n", i);
      error = 1;
      return;
    }
    if (fscanf(f, "%s", colorString) == EOF) {
      printf("File invalid near a color name in segment %d.\n, i");
      error = 1;
      return;
    }
    data.pic[i].color = ColornameToInt(colorString);
    if (data.pic[i].color == BAD_COLORNAME) {
      printf("Invalid color name in segment %d.\n", i);
      error = 1;
      return;
    }
    for (j=0; j < data.pic[i].size; j++) {
      if (fscanf(f, " (%d,%d)", &x, &y) == EOF) {
        printf("File invalid near coordinate pair %d in segment %d.\n",j,i);
        error = 1;
        return;
      }
      data.pic[i].points[j].x = convertIn(x, X);
      data.pic[i].points[j].y = convertIn(y, Y);
    }
  }
  fclose(f);
}

void load_mud(filename)
char *filename;
{
   FILE *f;
   char c;
   int  i, j, cnt;
   unsigned char *s;

   f = fopen(filename,"r");
   if (!f) {
     printf("Cannot read file, Sorry!\n");
     error=1;
     return;
   }
   s = (unsigned char *) 
	 malloc(sizeof(char)*(3+MAXDESCSIZE*(4+4*MAXPOINTPOLY)));
   if (FindGraph(f)) {
       while((c=fgetc(f)) != '"');
       i = 0;
       for(c=fgetc(f); ((c != EOF) && c !=';') ; c=fgetc(f))
           if (ishex(c)) 
	      s[i++]=c;
   }
   else {
      printf("File doesn't contain the 'graph' function.\n");
      error = 1;
      return;
   }
   s[i]='\0';
   data.cPoly = HexToInt(s[0], s[1]);
   printf("Reading %d polygons from file.\n",data.cPoly);
   cnt = 2;
   for (i=0; i<data.cPoly; i++) {
     data.pic[i].size = HexToInt(s[cnt], s[cnt+1]);
     data.pic[i].color = HexToInt(s[cnt+2], s[cnt+3]);
     cnt += 4;
     for (j=0; j < data.pic[i].size; j++) {
       data.pic[i].points[j].x = convertIn(HexToInt(s[cnt], s[cnt+1]), X);
       data.pic[i].points[j].y = convertIn(HexToInt(s[cnt+2], s[cnt+3]), Y);
       cnt += 4;
     }
   }
   fclose(f);
   free(s);
}

void load_file(filename)
char *filename;
{
   int type;

   error = 0;
   if (filename == NULL) {
      printf("No file name, file not loaded.\n");
      return;
   }
   type = findType(filename);
   switch (type) {
      case PPF:
	load_ppf(filename);
	break;
      case BSX:
        load_bsx(filename); 
	break;
      case MUD:
        load_mud(filename); 
	break;
      case XPF:
	load_xpf(filename);
	break;
      case BAD_FILE_TYPE:
      default:
        printf("Illegal file extention.\n");
   }
   CheckIntegrity();
   if (error) {
      printf("Error loading file.\n");
      data.cPoly = 0;
   }
   UpdateUndo();
   SetPolyNumLabel();
}
