#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/cursorfont.h>
#include <X11/Xaw/XawInit.h>
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/SmeBSB.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include "config.h"
#include "bitmaps/mud.icon"

extern void HandlePress();
extern void HandleMove();
extern void ButtonCallback();
extern void ColorCallback();
extern void ExposeCallback();
extern void PrimitiveCallback();

Widget toplevel, canvas, menuButton, polyNum;
Widget fileText, fileShell = NULL;
int    LoadOrSave;

void GetName();

XtActionsRec actionTable[] = {
    {"get-name", GetName}
};

static String translations = 
   "#override  \n\
    Ctrl<Key>M:    no-op(RingBell)\n\
    <Key>Return:   get-name()";

static String Toggletrans = 
   "#override  \n\
    <EnterWindow>:        highlight(Always)\n\
    <LeaveWindow>:        unhighlight()\n\
    <Btn1Down>,<Btn1Up>:  set() notify()";

MakeIcon (w)
Widget w; 
{
    Pixmap icon;

    icon = XCreateBitmapFromData(XtDisplay(w),
				       RootWindowOfScreen(XtScreen(w)),
				       muddraw_bits, 
				       muddraw_width, muddraw_height);
    XtVaSetValues(w, XtNiconPixmap, icon, NULL);
}

Widget CreateShellWidget(argc, argv)
int  argc;
char **argv;
{
    toplevel = XtInitialize(argv[0], "Muddraw", NULL, 0, 
                            &argc, argv);
    XtAddActions(actionTable, XtNumber(actionTable));

    return toplevel;
}

Widget CreatePane(parent)
Widget parent;
{
    Widget pane;

    pane = XtCreateManagedWidget ("pane", panedWidgetClass,
                                  parent, NULL, 0);
    return pane;
}

Widget CreateBox(parent)
Widget parent;
{
   Widget box;
   Arg    wargs[3];
   int    n;

   n = 0;
   XtSetArg (wargs[n], XtNorientation, XtorientHorizontal); n++;
   XtSetArg (wargs[n], XtNshowGrip, False);                 n++;
   box = XtCreateManagedWidget("box", boxWidgetClass, parent, wargs, n);

   return box;
}

CreateRadioButton(parent, name, actionNum)
Widget parent;
char   *name;
int    actionNum;
{
   XtTranslations table;
   static Widget GroupWidget = NULL;
   Widget button;
   Arg    wargs[4];
   int    n;

   table = XtParseTranslationTable(Toggletrans);
   n = 0;
   XtSetArg (wargs[n], XtNlabel, name);                n++;
   if (GroupWidget != NULL) {
      XtSetArg (wargs[n], XtNradioGroup, GroupWidget); n++;
   }
   else {
      XtSetArg (wargs[n], XtNstate, True);             n++;
   }
   XtSetArg(wargs[n], XtNtranslations, table);         n++;
   button = XtCreateManagedWidget(name, toggleWidgetClass, parent, wargs, n);
   XtAddCallback (button, XtNcallback, PrimitiveCallback, actionNum); 
   if (GroupWidget == NULL) 
      GroupWidget = button;
}

CreateOptionButton(parent, name, actionNum)
Widget parent;
char   *name;
int    actionNum;
{
   Widget button;
   Arg    wargs[2];
   int    n;

   n = 0;
   XtSetArg (wargs[n], XtNlabel, name);              n++;
   button = XtCreateManagedWidget(name, commandWidgetClass, parent, wargs, n);
   XtAddCallback (button, XtNcallback, ButtonCallback, actionNum);
}

CreatePolyNumLabel(parent)
Widget parent;
{
   polyNum = XtCreateManagedWidget("polyNum", labelWidgetClass, 
				   parent, NULL, 0);
   SetPolyNumLabel();
}

Widget CreateColorBox(parent)
Widget parent;
{
   Widget box;
   Arg    wargs[4];
   int    n;

   n = 0;
#ifdef COLORS_256
   XtSetArg (wargs[n], XtNwidth, COLOR_BOX_X);       n++; 
   XtSetArg (wargs[n], XtNheight, COLOR_BOX_Y);      n++; 
#else
   XtSetArg (wargs[n], XtNorientation, XtorientVertical); n++;
#endif
   box = XtCreateManagedWidget("box", boxWidgetClass, parent, wargs, n);

   return box;
}

Widget CreateColorButtons(parent)
Widget parent;
{
   Widget button;
   Pixel  GetColor();
   Pixmap GetStipple();
   Arg    wargs[4];
   int    n, i;


   for (i = 0; i < NUM_COLORS; i++) {
      n = 0;
      XtSetArg (wargs[n], XtNwidth, COLOR_BUTTON_X); n++;
      XtSetArg (wargs[n], XtNheight, COLOR_BUTTON_Y); n++;
      XtSetArg (wargs[n], XtNlabel, " "); n++;
      if (isMono()) {
	  XtSetArg (wargs[n], XtNbackgroundPixmap, GetPixmap(i)); n++;
      }
      else {
	  XtSetArg (wargs[n], XtNbackground, GetColor(i)); n++;
      }
      button = XtCreateManagedWidget("color", commandWidgetClass, 
                                     parent, wargs, n);
      XtAddCallback (button, XtNcallback, ColorCallback, i); 
   }
}
   
Widget CreateMenuButton(parent)
Widget parent;
{
   Arg    wargs[2];
   int    n;
 
   n = 0;
   XtSetArg (wargs[n], XtNlabel, "Polygon"); n++;
   XtSetArg (wargs[n], XtNmenuName, "primitiveMenu"); n++;
   menuButton = XtCreateManagedWidget("menu", menuButtonWidgetClass, 
		                      parent, wargs, n);
   return menuButton;
}


void CreateOption(parent, name, actionNum)
Widget parent;
char   *name;
int    actionNum;
{
   Widget w;
   Arg    wargs[2];
   int    n;

   n=0;
   XtSetArg (wargs[n], XtNlabel, name); n++;
   w = XtCreateManagedWidget ("smeBSB", smeBSBObjectClass,
                              parent, wargs, n);

   XtAddCallback (w, XtNcallback, PrimitiveCallback, actionNum); 
}

void CreateOptionMenu(parent)
Widget parent;
{
   Widget primitiveMenu;
   Arg    wargs[2];
   int    n;
 
   primitiveMenu = XtCreatePopupShell ("primitiveMenu", simpleMenuWidgetClass,
				       parent, NULL, 0);

   CreateOption(primitiveMenu, "Polygon", 1); 
   CreateOption(primitiveMenu, "Rectangle", 2); 
   CreateOption(primitiveMenu, "Ellipse", 3); 
   CreateOption(primitiveMenu, "Move", 4); 
}

Widget CreateCanvas(parent, width, height)
Widget parent;
int width, height;
{
    Arg     wargs[4];
    int n;

    /* Create a drawing surface */
    n=0;
    XtSetArg(wargs[n], XtNwidth, width);                          n++; 
    XtSetArg(wargs[n], XtNheight, height);                        n++; 
    XtSetArg(wargs[n], XtNcursor, 
	     XCreateFontCursor(XtDisplay(parent), XC_crosshair)); n++;
    XtSetArg(wargs[n], XtNbackground, GetColor(0));               n++; 
    canvas = XtCreateManagedWidget("canvas", 
                                   simpleWidgetClass, 
                                   parent, wargs, n);
    XtAddEventHandler (canvas, ExposureMask, FALSE, ExposeCallback, NULL); 
    return canvas;
}

void AddEventHandlers(w)
Widget w;
{
    XtAddEventHandler(w, ButtonPressMask, FALSE,
                      HandlePress, NULL);
    XtAddEventHandler(w, PointerMotionMask, FALSE,
                      HandleMove, NULL);
}

void GrabPointer()
{
   XGrabPointer(XtDisplay(canvas), XtWindow(canvas), True, 
		ButtonPressMask | PointerMotionMask,
		GrabModeAsync, GrabModeAsync,
		XtWindow(canvas),
		None, CurrentTime); 
}

void UngrabPointer()
{  
   XUngrabPointer(XtDisplay(canvas), CurrentTime);
}

void MoveWidgetToCursor(w)
Widget w;
{
   Arg    wargs[2];
   Window root, child;
   int n, x, y, win_x, win_y;
   unsigned int key_buttons;

   XQueryPointer(XtDisplay(w), XtWindow(w),
		 &root, &child,
		 &x, &y, &win_x, &win_y,
                 &key_buttons);

   n=0;
   XtSetArg(wargs[n], XtNx, x); n++;
   XtSetArg(wargs[n], XtNy, y); n++;
   XtSetValues(w, wargs, n);
}

void DoFileOpp()
{
   Arg  wargs[1];
   int  n;
   char *localS;

   n = 0;
   XtSetArg(wargs[n], XtNstring, &localS);  n++;
   XtGetValues (fileText, wargs, n);

   if (LoadOrSave == 1) { 
      load_file(localS);
      ClearPicture();
      RedrawPicture();
   }
   else
      save_file(localS);
   XtPopdown(fileShell);
}

void CancelFileLoad(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
   XtPopdown(fileShell);
}

void okFileLoad(w, client_data, call_data)
Widget w;
caddr_t client_data;
caddr_t call_data;
{
    DoFileOpp();
}

void GetName(w, e, s, num)
Widget w;
XEvent *e;
String *s;
Cardinal *num;
{
    DoFileOpp();
}

MakeFilenamePopup()
{
   XtTranslations table;
   Widget  filePopup, okButton, cancelButton;
   Arg     wargs[7];
   int     n;

   n = 0;
   fileShell = XtCreatePopupShell("Filename", transientShellWidgetClass,
                                  toplevel, NULL, 0);
   n = 0;
   filePopup = XtCreateManagedWidget("popup", panedWidgetClass, fileShell,
                                     NULL, 0);

   table = XtParseTranslationTable(translations);
   n = 0;
   XtSetArg(wargs[n], XtNshowGrip, False);           n++;
   XtSetArg(wargs[n], XtNeditType, XawtextEdit);     n++;
   XtSetArg(wargs[n], XtNtranslations, table);       n++;
   fileText = XtCreateManagedWidget("fileMenu", asciiTextWidgetClass, 
                         filePopup, wargs, n);

   n = 0;
   XtSetArg(wargs[n], XtNshowGrip, False);  n++;
   XtSetArg(wargs[n], XtNresize, False);    n++;
   XtSetArg(wargs[n], XtNlabel, "OK");      n++;
   okButton = XtCreateManagedWidget("okButton", commandWidgetClass,
                                        filePopup, wargs, n);
   n = 0;
   XtSetArg(wargs[n], XtNresize, False);    n++;
   XtSetArg(wargs[n], XtNlabel, "CANCEL");  n++;
   cancelButton = XtCreateManagedWidget("cancelButton", commandWidgetClass,
                                        filePopup, wargs, n);

   XtAddCallback(okButton, XtNcallback, okFileLoad, NULL); 
   XtAddCallback(cancelButton, XtNcallback, CancelFileLoad, NULL); 
   XtPopup(fileShell, XtGrabNonexclusive);
   MoveWidgetToCursor(fileShell);
}

void GetFileName(operation)
int operation;
{
   LoadOrSave = operation;
   if (fileShell == NULL) 
      MakeFilenamePopup();
   else {
      XtPopup(fileShell, XtGrabNonexclusive);
      MoveWidgetToCursor(fileShell);
   }
}
